@@ -5,6 +5,7 @@ pub mod data;
 pub mod decode;
 pub mod derive;
 pub mod errors;
+pub mod find;
 pub mod limiter;
 pub mod mint;
 pub mod opt;
@@ -14,4 +15,6 @@ pub mod sign;
 pub mod snapshot;
 pub mod spinner;
 pub mod update_metadata;
+pub mod uses;
+pub mod utils;
 pub mod withdraw;
