@@ -8,7 +8,7 @@ pub struct Opt {
     pub rpc: Option<String>,
 
     /// Timeout to override default value of 60 seconds
-    #[structopt(short, long, global = true, default_value = "90")]
+    #[structopt(short = "T", long, global = true, default_value = "90")]
     pub timeout: u64,
 
     /// Log level
@@ -27,6 +27,12 @@ pub enum Command {
         #[structopt(subcommand)]
         collections_subcommands: CollectionsSubcommands,
     },
+    /// NFT uses commands
+    #[structopt(name = "uses")]
+    Uses {
+        #[structopt(subcommand)]
+        uses_subcommands: UsesSubcommands,
+    },
     /// Burn token
     #[structopt(name = "burn")]
     Burn {
@@ -44,6 +50,12 @@ pub enum Command {
         #[structopt(subcommand)]
         derive_subcommands: DeriveSubcommands,
     },
+    /// Find things.
+    #[structopt(name = "find")]
+    Find {
+        #[structopt(subcommand)]
+        find_subcommands: FindSubcommands,
+    },
     /// Mint new NFTs from JSON files
     #[structopt(name = "mint")]
     Mint {
@@ -83,11 +95,12 @@ pub enum Command {
 
 #[derive(Debug, StructOpt)]
 pub enum BurnSubcommands {
+    /// Burn one NFT.
     #[structopt(name = "one")]
     One {
         /// Path to the authority & funder keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Token mint account
         #[structopt(short, long)]
@@ -96,12 +109,151 @@ pub enum BurnSubcommands {
 }
 
 #[derive(Debug, StructOpt)]
-pub enum CollectionsSubcommands {
+pub enum UsesSubcommands {
+    /// Approve a delegate authority that is allowed to make changes to the NFT's Use data.
     #[structopt(name = "approve-authority")]
     ApproveAuthority {
         /// Path to the update authority keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
+
+        /// NFT mint address
+        #[structopt(short, long)]
+        mint_nft: String,
+
+        /// Delegate use authority address
+        #[structopt(short, long)]
+        delegate_use_authority: String,
+
+        /// NFT Owner Token Account
+        #[structopt(short, long)]
+        token_account_nft: String,
+
+        /// Burner Program ID
+        #[structopt(short, long)]
+        burner_program_id: String,
+
+        /// Number of uses
+        #[structopt(short, long)]
+        number_of_uses: u64,
+    },
+    /// Revoke a delegate authority from being allowed to make changes to the NFT's Use data.
+    #[structopt(name = "revoke-authority")]
+    RevokeAuthority {
+        /// Path to the update authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
+
+        /// NFT mint address
+        #[structopt(short, long)]
+        mint_nft: String,
+
+        /// Delegate use authority address
+        #[structopt(short, long)]
+        delegate_use_authority: String,
+
+        /// NFT Owner Token Account
+        #[structopt(short, long)]
+        token_account_nft: String,
+    },
+    /// Use a NFT, following the on-chain logic for burning it if set.
+    #[structopt(name = "utilize")]
+    Utilize {
+        /// Path to the use authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
+
+        /// NFT mint address
+        #[structopt(short, long)]
+        mint_nft: String,
+
+        /// NFT Holder/Owner address
+        #[structopt(short, long)]
+        holder_nft: String,
+
+        /// NFT Owner Token Account
+        #[structopt(short, long)]
+        token_account_nft: String,
+
+        /// Optional Burner Program ID (if token use is set to Burn)
+        #[structopt(short, long)]
+        burner_program_id: Option<String>,
+
+        /// Option if the signing keypair is a use authority delegate.
+        #[structopt(short = "d", long)]
+        is_delegate: bool,
+    },
+}
+
+#[derive(Debug, StructOpt)]
+pub enum CollectionsSubcommands {
+    /// Verify collection on an NFT.
+    #[structopt(name = "verify")]
+    VerifyCollection {
+        /// Path to the update authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
+
+        /// Collection mint address
+        #[structopt(short, long)]
+        collection_mint: String,
+
+        /// NFT mint address
+        #[structopt(short = "n", long)]
+        nft_mint: String,
+
+        /// Option if the signing keypair is a collection authority delegate.
+        #[structopt(short = "d", long)]
+        is_delegate: bool,
+    },
+    /// Set an NFT's collection as unverified.
+    #[structopt(name = "unverify")]
+    UnverifyCollection {
+        /// Path to the update authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
+
+        /// Collection mint address
+        #[structopt(short, long)]
+        collection_mint: String,
+
+        /// NFT mint address
+        #[structopt(short = "n", long)]
+        nft_mint: String,
+
+        /// Option if the signing keypair is a collection authority delegate.
+        #[structopt(short = "d", long)]
+        is_delegate: bool,
+    },
+    /// Set collection value on NFT and verify in the same step.
+    #[structopt(name = "set-and-verify")]
+    SetAndVerifyCollection {
+        /// Path to the collection update authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
+
+        /// Collection mint address
+        #[structopt(short, long)]
+        collection_mint: String,
+
+        /// NFT mint address
+        #[structopt(short = "n", long)]
+        nft_mint: String,
+
+        /// NFT update authority
+        #[structopt(short, long)]
+        update_authority_nft: String,
+
+        /// Option if the signing keypair is a collection authority delegate.
+        #[structopt(short = "d", long)]
+        is_delegate: bool,
+    },
+    /// Approve a delegate authority that is allowed to change collection data on the NFT.
+    #[structopt(name = "approve-authority")]
+    ApproveAuthority {
+        /// Path to the update authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
 
         /// Collection mint address
         #[structopt(short, long)]
@@ -111,11 +263,12 @@ pub enum CollectionsSubcommands {
         #[structopt(short, long)]
         delegate_authority: String,
     },
+    /// Revoke the delegate authority from being allowed to change collection data on the NFT.
     #[structopt(name = "revoke-authority")]
     RevokeAuthority {
         /// Path to the update authority keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Collection mint address
         #[structopt(short, long)]
@@ -152,6 +305,11 @@ pub enum DecodeSubcommands {
         #[structopt(short, long, default_value = ".")]
         output: String,
     },
+    /// Decode a mint account's master edition
+    Master {
+        #[structopt(short, long)]
+        account: String,
+    },
 }
 
 #[derive(Debug, StructOpt)]
@@ -175,6 +333,16 @@ pub enum DeriveSubcommands {
     CMV2Creator { candy_machine_id: String },
 }
 
+#[derive(Debug, StructOpt)]
+pub enum FindSubcommands {
+    /// Find any missing editions for a Master NFT mint account.
+    #[structopt(name = "missing-editions")]
+    MissingEditions {
+        #[structopt(short, long)]
+        account: String,
+    },
+}
+
 #[derive(Debug, StructOpt)]
 pub enum MintSubcommands {
     /// Mint a single NFT from a JSON file
@@ -182,7 +350,7 @@ pub enum MintSubcommands {
     One {
         /// Path to the update_authority keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Receiving address, if different from update authority.
         #[structopt(short = "R", long)]
@@ -204,16 +372,53 @@ pub enum MintSubcommands {
         #[structopt(short, long)]
         primary_sale_happened: bool,
 
+        /// Maximum number of editions. Defaults to zero, meaning no editions allowed.
+        #[structopt(short = "e", long, default_value = "0")]
+        max_editions: u64,
+
         /// Sign NFT after minting it
         #[structopt(long)]
         sign: bool,
     },
+    /// Mint one or more editions from a Master NFT.
+    #[structopt(name = "editions")]
+    Editions {
+        /// Path to the update_authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
+
+        /// Master Edition NFT mint account/token address.
+        #[structopt(short, long)]
+        account: String,
+
+        /// Receiving address, if different from update authority.
+        #[structopt(short = "R", long)]
+        receiver: Option<String>,
+
+        /// Mint the next n editions in order.
+        #[structopt(short, long)]
+        next_editions: Option<u64>,
+
+        /// Mint the provided specific editions e.g.: --specific-editions 1,7,10
+        #[structopt(short = "s", long)]
+        specific_editions: Option<Vec<u64>>,
+    },
+    /// Find any missing editions for a Master NFT.
+    #[structopt(name = "missing-editions")]
+    MissingEditions {
+        /// Path to the update_authority keypair file
+        #[structopt(short, long)]
+        keypair: Option<String>,
+
+        #[structopt(short, long)]
+        account: String,
+    },
     #[structopt(name = "list")]
     /// Mint a list of NFTs from a directory of JSON files
     List {
         /// Path to the update_authority keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Receiving address, if different from update authority
         #[structopt(short = "R", long)]
@@ -238,6 +443,12 @@ pub enum MintSubcommands {
         /// Sign NFTs after minting them
         #[structopt(long)]
         sign: bool,
+
+        /// Track whether URIs are succesfully minted or not, and output to
+        /// `minted` file with URIs -> Mint Accounts or to `unminted` file
+        /// with list of unminted URIs for easy continuation of command
+        #[structopt(long)]
+        track: bool,
     },
 }
 
@@ -248,7 +459,7 @@ pub enum SetSubcommands {
     PrimarySaleHappened {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -259,7 +470,7 @@ pub enum SetSubcommands {
     UpdateAuthority {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -268,13 +479,17 @@ pub enum SetSubcommands {
         /// New update authority address
         #[structopt(short = "u", long)]
         new_update_authority: String,
+
+        //Path to the payers's keypair file
+        #[structopt(short = "p", long)]
+        keypair_payer: Option<String>,
     },
     /// Set update authority on multiple accounts to a new account
     #[structopt(name = "update-authority-all")]
     UpdateAuthorityAll {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Path to JSON mint accounts file
         #[structopt(short = "a", long)]
@@ -283,13 +498,17 @@ pub enum SetSubcommands {
         /// New update authority address
         #[structopt(short = "u", long)]
         new_update_authority: String,
+
+        //Path to the payers's keypair file
+        #[structopt(short = "p", long)]
+        keypair_payer: Option<String>,
     },
     /// Set is-mutable to false, preventing any future updates to the NFT
     #[structopt(name = "immutable")]
     Immutable {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -298,7 +517,7 @@ pub enum SetSubcommands {
     ImmutableAll {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Path to JSON mint accounts file
         #[structopt(short, long)]
@@ -313,7 +532,7 @@ pub enum SignSubcommands {
     One {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account to sign
         #[structopt(short, long)]
@@ -324,7 +543,7 @@ pub enum SignSubcommands {
     All {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Creator to filter accounts by (for CM v2 use --v2 if candy_machine account is passed)
         #[structopt(short, long)]
@@ -416,7 +635,7 @@ pub enum UpdateSubcommands {
     Name {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -431,7 +650,7 @@ pub enum UpdateSubcommands {
     Symbol {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -446,7 +665,7 @@ pub enum UpdateSubcommands {
     Creators {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -457,7 +676,7 @@ pub enum UpdateSubcommands {
         new_creators: String,
 
         /// Should be appended instead of overwriting
-        #[structopt(short, long = "append")]
+        #[structopt(short = "A", long = "append")]
         append: bool,
     },
     /// Update the data struct on a NFT
@@ -465,7 +684,7 @@ pub enum UpdateSubcommands {
     Data {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -480,7 +699,7 @@ pub enum UpdateSubcommands {
     DataAll {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Path to directory containing JSON files with new data
         #[structopt(short, long)]
@@ -491,7 +710,7 @@ pub enum UpdateSubcommands {
     Uri {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// Mint account of corresponding metadata to update
         #[structopt(short, long)]
@@ -506,7 +725,7 @@ pub enum UpdateSubcommands {
     UriAll {
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
 
         /// JSON file with list of mint accounts and new URIs
         #[structopt(short = "u", long)]
@@ -524,6 +743,6 @@ pub enum WithdrawSubcommands {
 
         /// Path to the creator's keypair file
         #[structopt(short, long)]
-        keypair: String,
+        keypair: Option<String>,
     },
 }
